@@ -34,8 +34,9 @@ from nautilus_trader.common.logging import LogColor
 from nautilus_trader.common.logging import Logger
 from nautilus_trader.common.timer import TimeEvent
 from nautilus_trader.core.datetime import millis_to_nanos
-from nautilus_trader.execution.messages import ExecutionReport
-from nautilus_trader.execution.messages import OrderStatusReport
+from nautilus_trader.execution.reports import OrderStatusReport
+from nautilus_trader.execution.reports import PositionStatusReport
+from nautilus_trader.execution.reports import TradeReport
 from nautilus_trader.model.commands.trading import CancelAllOrders
 from nautilus_trader.model.commands.trading import CancelOrder
 from nautilus_trader.model.commands.trading import ModifyOrder
@@ -43,22 +44,26 @@ from nautilus_trader.model.commands.trading import SubmitOrder
 from nautilus_trader.model.commands.trading import SubmitOrderList
 from nautilus_trader.model.enums import AccountType
 from nautilus_trader.model.enums import LiquiditySide
+from nautilus_trader.model.enums import OMSType
+from nautilus_trader.model.enums import OrderSide
 from nautilus_trader.model.enums import OrderSideParser
 from nautilus_trader.model.enums import OrderType
+from nautilus_trader.model.enums import PositionSide
 from nautilus_trader.model.enums import PositionSideParser
 from nautilus_trader.model.enums import TimeInForce
 from nautilus_trader.model.enums import TimeInForceParser
-from nautilus_trader.model.enums import VenueType
 from nautilus_trader.model.events.account import AccountState
 from nautilus_trader.model.identifiers import AccountId
 from nautilus_trader.model.identifiers import ClientId
 from nautilus_trader.model.identifiers import ClientOrderId
-from nautilus_trader.model.identifiers import ExecutionId
 from nautilus_trader.model.identifiers import InstrumentId
 from nautilus_trader.model.identifiers import StrategyId
-from nautilus_trader.model.identifiers import Symbol
+from nautilus_trader.model.identifiers import TradeId
+from nautilus_trader.model.identifiers import Venue
 from nautilus_trader.model.identifiers import VenueOrderId
 from nautilus_trader.model.instruments.base import Instrument
+
+# from nautilus_trader.model.objects import MarginBalance
 from nautilus_trader.model.objects import AccountBalance
 from nautilus_trader.model.objects import Money
 from nautilus_trader.model.objects import Price
@@ -148,9 +153,8 @@ class BinanceExecutionClient(LiveExecutionClient):
         super().__init__(
             loop=loop,
             client_id=ClientId(name),
+            oms_type=OMSType.NETTING,
             instrument_provider=instrument_provider,
-            venue_type=VenueType.EXCHANGE,
-            account_id=account_id,
             account_type=account_type,
             base_currency=None,
             msgbus=msgbus,
@@ -160,6 +164,9 @@ class BinanceExecutionClient(LiveExecutionClient):
             config={"name": f"BinanceExecClient-{name}"},
         )
 
+        self._set_account_id(account_id)
+        self._set_venue(Venue(account_id.issuer))
+
         self._client = client
 
         # Hot caches
@@ -221,7 +228,7 @@ class BinanceExecutionClient(LiveExecutionClient):
         # Connect WebSocket clients
         self._ws_user_api.subscribe(key=self._listen_key)
         await self._ws_user_api.connect()
-        self._ws_user_api.set_reconnect_callback(self._sync_account_state)
+        self._ws_user_api.add_after_connect_callback(self._sync_account_state)
 
         self._set_connected(True)
         self._log.info("Connected.")
@@ -239,7 +246,12 @@ class BinanceExecutionClient(LiveExecutionClient):
         pass
 
     async def _sync_account_state(self):
-        response: Dict[str, Any] = await self._account_api.account(recv_window=5000)
+        try:
+            response: Dict[str, Any] = await self._account_api.account(recv_window=10000)
+        except BinanceError as ex:
+            self._log.error(ex.message)
+            return
+
         self._update_account_state(response=response)
 
     def _authenticate_api_key(self, response: Dict[str, Any]) -> None:
@@ -303,56 +315,122 @@ class BinanceExecutionClient(LiveExecutionClient):
                 self._log.debug(f"Currency {sym} price not found")
         return equity
 
-    # -- RECONCILIATION ----------------------------------------------------------------------------
+    # -- EXECUTION REPORTS -------------------------------------------------------------------------
 
-    async def generate_order_status_report(self, order: Order) -> OrderStatusReport:  # type: ignore
+    async def generate_order_status_report(
+        self,
+        venue_order_id: VenueOrderId = None,
+    ) -> Optional[OrderStatusReport]:
         """
-        Generate an order status report for the given order.
+        Generate an order status report for the given venue order ID.
 
-        If an error occurs then logs and returns ``None``.
+        If the order is not found, or an error occurs, then logs and returns
+        ``None``.
 
         Parameters
         ----------
-        order : Order
-            The order for the report.
+        venue_order_id : VenueOrderId, optional
+            The venue order ID (assigned by the venue) query filter.
 
         Returns
         -------
         OrderStatusReport or ``None``
 
         """
-        self._log.error(  # pragma: no cover
-            "Cannot generate order status report: not yet implemented.",
-        )
+        self._log.warning("Cannot generate OrderStatusReport: not yet implemented.")
 
-    async def generate_exec_reports(
+        return None
+
+    async def generate_order_status_reports(
         self,
-        venue_order_id: VenueOrderId,
-        symbol: Symbol,
-        since: datetime = None,
-    ) -> List[ExecutionReport]:  # type: ignore
+        instrument_id: InstrumentId = None,
+        start: datetime = None,
+        end: datetime = None,
+        open_only: bool = False,
+    ) -> List[OrderStatusReport]:
         """
-        Generate a list of execution reports.
+        Generate a list of order status reports with optional query filters.
+
+        The returned list may be empty if no orders match the given parameters.
+
+        Parameters
+        ----------
+        instrument_id : InstrumentId, optional
+            The instrument ID query filter.
+        start : datetime, optional
+            The start datetime query filter.
+        end : datetime, optional
+            The end datetime query filter.
+        open_only : bool, default False
+            If the query is for open orders only.
+
+        Returns
+        -------
+        list[OrderStatusReport]
+
+        """
+        self._log.warning("Cannot generate OrderStatusReports: not yet implemented.")
+
+        return []
+
+    async def generate_trade_reports(
+        self,
+        instrument_id: InstrumentId = None,
+        venue_order_id: VenueOrderId = None,
+        start: datetime = None,
+        end: datetime = None,
+    ) -> List[TradeReport]:
+        """
+        Generate a list of trade reports with optional query filters.
 
         The returned list may be empty if no trades match the given parameters.
 
         Parameters
         ----------
-        venue_order_id : VenueOrderId
-            The venue order ID for the trades.
-        symbol : Symbol
-            The symbol for the trades.
-        since : datetime, optional
-            The timestamp to filter trades on.
+        instrument_id : InstrumentId, optional
+            The instrument ID query filter.
+        venue_order_id : VenueOrderId, optional
+            The venue order ID (assigned by the venue) query filter.
+        start : datetime, optional
+            The start datetime query filter.
+        end : datetime, optional
+            The end datetime query filter.
 
         Returns
         -------
-        list[ExecutionReport]
+        list[TradeReport]
 
         """
-        self._log.error(  # pragma: no cover
-            "Cannot generate execution report: not yet implemented.",
-        )
+        self._log.warning("Cannot generate TradeReports: not yet implemented.")
+
+        return []
+
+    async def generate_position_status_reports(
+        self,
+        instrument_id: InstrumentId = None,
+        start: datetime = None,
+        end: datetime = None,
+    ) -> List[PositionStatusReport]:
+        """
+        Generate a list of position status reports with optional query filters.
+
+        The returned list may be empty if no positions match the given parameters.
+
+        Parameters
+        ----------
+        instrument_id : InstrumentId, optional
+            The instrument ID query filter.
+        start : datetime, optional
+            The start datetime query filter.
+        end : datetime, optional
+            The end datetime query filter.
+
+        Returns
+        -------
+        list[PositionStatusReport]
+
+        """
+        self._log.warning("Cannot generate PositionStatusReports: not yet implemented.")
 
         return []
 
@@ -411,13 +489,14 @@ class BinanceSpotExecutionClient(BinanceExecutionClient):
 
         self.generate_account_state(
             balances=balances,
+            margins=[],
             reported=True,
             ts_event=millis_to_nanos(response["updateTime"]),
         )
 
     async def _take_snapshot(self):
         try:
-            response: Dict[str, Any] = await self._account_api.account(recv_window=5000)
+            response: Dict[str, Any] = await self._account_api.account(recv_window=10000)
             balances = parse_balances(
                 self._instrument_provider, response["balances"], "asset", "free", "locked"
             )
@@ -426,8 +505,8 @@ class BinanceSpotExecutionClient(BinanceExecutionClient):
 
             balance_equity = await self._calculate_balance_equity(balances)
             self._log.debug(f"Balance: {balances}")
-        except Exception as ex:
-            self._log.exception(ex)
+        except BinanceError as ex:
+            self._log.error(ex.message)
             return
 
         account_state: AccountState = AccountState(
@@ -436,7 +515,12 @@ class BinanceSpotExecutionClient(BinanceExecutionClient):
             base_currency=self.base_currency,
             reported=True,
             balances=balances,
-            info={"positions": [], "equity": float(balance_equity)},
+            margins=[],
+            info={
+                "positions": [],
+                "equity": float(balance_equity),
+                "equities": [{"currency": "USDT", "total": float(balance_equity)}],
+            },
             event_id=self._uuid_factory.generate(),
             ts_event=self._clock.timestamp_ns(),  # binance account updateTime is last change time
             ts_init=self._clock.timestamp_ns(),
@@ -467,6 +551,7 @@ class BinanceSpotExecutionClient(BinanceExecutionClient):
 
         self.generate_account_state(
             balances=balances,
+            margins=[],
             reported=True,
             ts_event=millis_to_nanos(data["u"]),
         )
@@ -532,7 +617,7 @@ class BinanceSpotExecutionClient(BinanceExecutionClient):
                 client_order_id=client_order_id,
                 venue_order_id=venue_order_id,
                 venue_position_id=None,  # NETTING accounts
-                execution_id=ExecutionId(str(data["t"])),  # Trade ID
+                trade_id=TradeId(str(data["t"])),  # Trade ID
                 order_side=OrderSideParser.from_str_py(data["S"]),
                 order_type=parse_order_type(order_type_str),
                 last_qty=Quantity.from_str(data["l"]),
@@ -612,7 +697,7 @@ class BinanceSpotExecutionClient(BinanceExecutionClient):
                 strategy_id=command.strategy_id,
                 instrument_id=command.instrument_id,
                 client_order_id=command.order.client_order_id,
-                reason=str(ex),  # type: ignore  # TODO(cs): Improve errors
+                reason=ex.message,  # type: ignore  # TODO(cs): Improve errors
                 ts_event=self._clock.timestamp_ns(),  # TODO(cs): Parse from response
             )
 
@@ -662,7 +747,7 @@ class BinanceSpotExecutionClient(BinanceExecutionClient):
             time_in_force=TimeInForceParser.to_str_py(order.time_in_force),
             quantity=str(order.quantity),
             price=str(order.price),
-            stop_price=str(order.trigger),
+            stop_price=str(order.trigger_price),
             iceberg_qty=str(order.display_qty) if order.display_qty is not None else None,
             new_client_order_id=order.client_order_id.value,
             recv_window=5000,
@@ -676,17 +761,53 @@ class BinanceSpotExecutionClient(BinanceExecutionClient):
     async def _cancel_order(self, command: CancelOrder) -> None:
         self._log.debug(f"Canceling order {command.client_order_id.value}.")
 
+        self.generate_order_pending_cancel(
+            strategy_id=command.strategy_id,
+            instrument_id=command.instrument_id,
+            client_order_id=command.client_order_id,
+            venue_order_id=command.venue_order_id,
+            ts_event=self._clock.timestamp_ns(),
+        )
+
         try:
             await self._account_api.cancel_order(
                 symbol=command.instrument_id.symbol.value,
                 orig_client_order_id=command.client_order_id.value,
             )
         except BinanceError as ex:
-            self._log.error(str(ex))  # type: ignore  # TODO(cs): Improve errors
+            self._log.error(ex.message)  # type: ignore  # TODO(cs): Improve errors
 
     async def _cancel_all_orders(self, command: CancelAllOrders) -> None:
         self._log.debug(f"Canceling all orders for {command.instrument_id.value}.")
 
+        # Cancel all in-flight orders
+        inflight_orders = self._cache.orders_inflight(
+            instrument_id=command.instrument_id,
+            strategy_id=command.strategy_id,
+        )
+        for order in inflight_orders:
+            self.generate_order_pending_cancel(
+                strategy_id=order.strategy_id,
+                instrument_id=order.instrument_id,
+                client_order_id=order.client_order_id,
+                venue_order_id=order.venue_order_id,
+                ts_event=self._clock.timestamp_ns(),
+            )
+
+        # Cancel all working orders
+        open_orders = self._cache.open_orders(
+            instrument_id=command.instrument_id,
+            strategy_id=command.strategy_id,
+        )
+        for order in open_orders:
+            self.generate_order_pending_cancel(
+                strategy_id=order.strategy_id,
+                instrument_id=order.instrument_id,
+                client_order_id=order.client_order_id,
+                venue_order_id=order.venue_order_id,
+                ts_event=self._clock.timestamp_ns(),
+            )
+
         try:
             await self._account_api.cancel_open_orders(
                 symbol=command.instrument_id.symbol.value,
@@ -745,6 +866,7 @@ class BinanceFutureExecutionClient(BinanceExecutionClient):
 
         self._account_api = account_api
         self._dual_side_position = False
+        self._position_side_by_client_order_id: Dict[ClientOrderId, PositionSide] = {}
 
     def _update_account_state(self, response: Dict[str, Any]) -> None:
         if self._snapshot_enabled:
@@ -763,6 +885,7 @@ class BinanceFutureExecutionClient(BinanceExecutionClient):
 
         self.generate_account_state(
             balances=balances,
+            margins=[],
             reported=True,
             ts_event=millis_to_nanos(response["updateTime"]),
         )
@@ -782,8 +905,8 @@ class BinanceFutureExecutionClient(BinanceExecutionClient):
 
             balance_equity = await self._calculate_balance_equity(balances)
             self._log.debug(f"Balance: {balances}")
-        except Exception as ex:
-            self._log.exception(ex)
+        except BinanceError as ex:
+            self._log.error(ex.message)
             return
 
         positions = parse_positions(response["positions"])
@@ -794,7 +917,12 @@ class BinanceFutureExecutionClient(BinanceExecutionClient):
             base_currency=self.base_currency,
             reported=True,
             balances=balances,
-            info={"positions": positions, "equity": float(balance_equity)},
+            margins=[],
+            info={
+                "positions": positions,
+                "equity": float(balance_equity),
+                "equities": [{"currency": "USDT", "total": float(balance_equity)}],
+            },
             event_id=self._uuid_factory.generate(),
             ts_event=self._clock.timestamp_ns(),  # binance account updateTime is last change time
             ts_init=self._clock.timestamp_ns(),
@@ -845,6 +973,7 @@ class BinanceFutureExecutionClient(BinanceExecutionClient):
 
         self.generate_account_state(
             balances=balances,
+            margins=[],
             reported=True,
             ts_event=ts_event,
         )
@@ -869,7 +998,7 @@ class BinanceFutureExecutionClient(BinanceExecutionClient):
 
             self.generate_reported_account(position_update, [], ts_event)
 
-    def _handle_execution_report(self, data: Dict[str, Any]):
+    def _handle_execution_report(self, data: Dict[str, Any]):  # noqa: C901
         ts_event = millis_to_nanos(data["E"])
         data = data["o"]
 
@@ -925,13 +1054,29 @@ class BinanceFutureExecutionClient(BinanceExecutionClient):
                 # Binance typically charges commission as base asset or BNB
                 commission = Money(0, instrument.base_currency)
 
+            venue_position_id = None
+            # TODO: Avoid same side order generate 2 different position_id
+            if (
+                self._dual_side_position
+                and client_order_id in self._position_side_by_client_order_id
+            ):
+                side = self._position_side_by_client_order_id[client_order_id]
+                open_positions = self._cache.positions_open(
+                    instrument_id=instrument_id, strategy_id=strategy_id
+                )
+                position = next(
+                    filter(lambda position: position.side == side, open_positions), None
+                )
+                if position is not None:
+                    venue_position_id = position.id
+
             self.generate_order_filled(
                 strategy_id=strategy_id,
                 instrument_id=instrument_id,
                 client_order_id=client_order_id,
                 venue_order_id=venue_order_id,
-                venue_position_id=None,
-                execution_id=ExecutionId(str(data["t"])),  # Trade ID
+                venue_position_id=venue_position_id,
+                trade_id=TradeId(str(data["t"])),  # Trade ID
                 order_side=OrderSideParser.from_str_py(data["S"]),
                 order_type=parse_future_order_type(order_type_str),
                 last_qty=Quantity.from_str(data["l"]),
@@ -976,20 +1121,22 @@ class BinanceFutureExecutionClient(BinanceExecutionClient):
     def cancel_all_orders(self, command: CancelAllOrders) -> None:
         self._loop.create_task(self._cancel_all_orders(command))
 
-    async def _submit_order(self, command: SubmitOrder) -> None:
+    async def _submit_order(self, command: SubmitOrder) -> None:  # noqa: C901
         self._log.debug(f"Submitting {command.order}.")
 
         position = None
         position_id = command.position_id
-        if position_id is None:
-            if self._dual_side_position:
-                self._log.error("Binance dual side position enabled, but not specify position side")
-                return
-        else:
+        if position_id is not None:
             position = self._cache.position(position_id)
             if position is None:
                 self._log.error(f"Position {position_id} not found")
                 return
+        elif self._dual_side_position:  # When not specify position_id, open position with same side
+            if command.order.side == OrderSide.BUY:
+                side = PositionSide.LONG
+            else:
+                side = PositionSide.SHORT
+            self._position_side_by_client_order_id[command.order.client_order_id] = side
 
         # Generate event here to ensure correct ordering of events
         self.generate_order_submitted(
@@ -1013,17 +1160,17 @@ class BinanceFutureExecutionClient(BinanceExecutionClient):
                 strategy_id=command.strategy_id,
                 instrument_id=command.instrument_id,
                 client_order_id=command.order.client_order_id,
-                reason=str(ex),  # type: ignore  # TODO(cs): Improve errors
+                reason=ex.message,  # type: ignore  # TODO(cs): Improve errors
                 ts_event=self._clock.timestamp_ns(),  # TODO(cs): Parse from response
             )
 
     def _position_kwargs(self, order: Order, position: Optional[Position] = None) -> Dict:
         if self._dual_side_position:
             if position is None:
-                # won't happen, design time error
-                return {}
-
-            return {"position_side": PositionSideParser.to_str_py(position.side)}
+                side = "LONG" if order.side == OrderSide.BUY else "SHORT"
+            else:
+                side = PositionSideParser.to_str_py(position.side)
+            return {"position_side": side}
         else:
             return {"position_side": "BOTH", "reduce_only": order.is_reduce_only}
 
@@ -1073,7 +1220,7 @@ class BinanceFutureExecutionClient(BinanceExecutionClient):
             time_in_force=TimeInForceParser.to_str_py(order.time_in_force),
             quantity=str(order.quantity),
             price=str(order.price),
-            stop_price=str(order.trigger),
+            stop_price=str(order.trigger_price),
             new_client_order_id=order.client_order_id.value,
             recv_window=5000,
             **self._position_kwargs(order, position),
@@ -1094,7 +1241,7 @@ class BinanceFutureExecutionClient(BinanceExecutionClient):
             type=binance_future_order_type(order=order, market_price=market_price),
             time_in_force=TimeInForceParser.to_str_py(order.time_in_force),
             quantity=str(order.quantity),
-            stop_price=str(order.trigger),
+            stop_price=str(order.trigger_price),
             new_client_order_id=order.client_order_id.value,
             recv_window=5000,
             **self._position_kwargs(order, position),
@@ -1108,17 +1255,53 @@ class BinanceFutureExecutionClient(BinanceExecutionClient):
     async def _cancel_order(self, command: CancelOrder) -> None:
         self._log.debug(f"Canceling order {command.client_order_id.value}.")
 
+        self.generate_order_pending_cancel(
+            strategy_id=command.strategy_id,
+            instrument_id=command.instrument_id,
+            client_order_id=command.client_order_id,
+            venue_order_id=command.venue_order_id,
+            ts_event=self._clock.timestamp_ns(),
+        )
+
         try:
             await self._account_api.cancel_order(
                 symbol=command.instrument_id.symbol.value,
                 orig_client_order_id=command.client_order_id.value,
             )
         except BinanceError as ex:
-            self._log.error(str(ex))  # type: ignore  # TODO(cs): Improve errors
+            self._log.error(ex.message)  # type: ignore  # TODO(cs): Improve errors
 
     async def _cancel_all_orders(self, command: CancelAllOrders) -> None:
         self._log.debug(f"Canceling all orders for {command.instrument_id.value}.")
 
+        # Cancel all in-flight orders
+        inflight_orders = self._cache.orders_inflight(
+            instrument_id=command.instrument_id,
+            strategy_id=command.strategy_id,
+        )
+        for order in inflight_orders:
+            self.generate_order_pending_cancel(
+                strategy_id=order.strategy_id,
+                instrument_id=order.instrument_id,
+                client_order_id=order.client_order_id,
+                venue_order_id=order.venue_order_id,
+                ts_event=self._clock.timestamp_ns(),
+            )
+
+        # Cancel all working orders
+        open_orders = self._cache.open_orders(
+            instrument_id=command.instrument_id,
+            strategy_id=command.strategy_id,
+        )
+        for order in open_orders:
+            self.generate_order_pending_cancel(
+                strategy_id=order.strategy_id,
+                instrument_id=order.instrument_id,
+                client_order_id=order.client_order_id,
+                venue_order_id=order.venue_order_id,
+                ts_event=self._clock.timestamp_ns(),
+            )
+
         try:
             await self._account_api.cancel_open_orders(
                 symbol=command.instrument_id.symbol.value,
