@@ -32,94 +32,26 @@ from nacre.adapters.zb.http.api.future_market import ZbFutureMarketHttpAPI
 from nacre.adapters.zb.http.api.spot_market import ZbSpotMarketHttpAPI
 from nacre.adapters.zb.http.client import ZbHttpClient
 from nacre.adapters.zb.http.error import ZbError
+from nacre.adapters.zb.http.spot import ZbSpotHttpClient
 from nacre.adapters.zb.parsing import parse_bar
 from nacre.adapters.zb.parsing import parse_bar_ws
 from nacre.adapters.zb.parsing import parse_book_snapshot_ws
 from nacre.adapters.zb.parsing import parse_diff_depth_stream_ws
 from nacre.adapters.zb.parsing import parse_mark_price
+from nacre.adapters.zb.parsing import parse_spot_bar
+from nacre.adapters.zb.parsing import parse_spot_book_snapshot_ws
+from nacre.adapters.zb.parsing import parse_spot_ticker_ws
+from nacre.adapters.zb.parsing import parse_spot_trade_tick_ws
 from nacre.adapters.zb.parsing import parse_ticker_ws
 from nacre.adapters.zb.parsing import parse_trade_tick
 from nacre.adapters.zb.parsing import parse_trade_tick_ws
 from nacre.adapters.zb.providers import ZbInstrumentProvider
-from nacre.adapters.zb.websocket.client import ZbWebSocketClient
 from nacre.adapters.zb.websocket.future import ZbFuturesWebSocket
 from nacre.adapters.zb.websocket.spot import ZbSpotWebSocket
 from nacre.model.data.tick import MarkTick
 
 
-# from nautilus_trader.adapters.binance.common import BINANCE_VENUE
-
-
 class ZbDataClient(LiveMarketDataClient):
-    """
-    Provides a data client for the Zb exchange.
-    """
-
-    def __init__(
-        self,
-        loop: asyncio.AbstractEventLoop,
-        client: ZbHttpClient,
-        msgbus: MessageBus,
-        cache: Cache,
-        clock: LiveClock,
-        logger: Logger,
-        venue: Venue,
-        market_api: ZbFutureMarketHttpAPI,
-        ws_api: ZbWebSocketClient,
-        instrument_provider: ZbInstrumentProvider,
-    ):
-        """
-        Initialize a new instance of the ``ZbDataClient`` class.
-
-        Parameters
-        ----------
-        loop : asyncio.AbstractEventLoop
-            The event loop for the client.
-        client : ZbHttpClient
-            The binance HTTP client.
-        msgbus : MessageBus
-            The message bus for the client.
-        cache : Cache
-            The cache for the client.
-        clock : LiveClock
-            The clock for the client.
-        logger : Logger
-            The logger for the client.
-        venue : Venue
-            The venue id for the client.
-        market_api : ZbFutureMarketHttpAPI
-            The market api
-        ws_api : ZbWebSocketClient
-            The websocket api
-        instrument_provider : ZbInstrumentProvider
-            The instrument provider.
-
-        """
-        super().__init__(
-            loop=loop,
-            client_id=ClientId(venue.value),
-            instrument_provider=instrument_provider,
-            msgbus=msgbus,
-            cache=cache,
-            clock=clock,
-            logger=logger,
-            config={"name": f"ZbDataClient-{venue.value}"},
-        )
-
-        self._client = client
-
-        self._update_instrument_interval: int = 60 * 60  # Once per hour (hardcode)
-        self._update_instruments_task: Optional[asyncio.Task] = None
-
-        # HTTP API
-        self._market_api = market_api
-        # WebSocket API
-        self._ws_api = ws_api
-
-        self._kline_buffer: Dict[InstrumentId, List[Any]] = {}
-        self._quote_tick_subscribed = False
-        self._last_quote_tick: Dict[InstrumentId, QuoteTick] = {}
-
     def connect(self):
         """
         Connect the client to Zb.
@@ -186,24 +118,6 @@ class ZbDataClient(LiveMarketDataClient):
 
     # -- SUBSCRIPTIONS -----------------------------------------------------------------------------
 
-    def subscribe(self, data_type: DataType) -> None:
-        if data_type.type == MarkTick:
-            instrument_id = data_type.metadata.get("instrument_id")
-            if instrument_id:
-                self._loop.create_task(self._subscribe_mark_price())
-                self._add_subscription(
-                    DataType(MarkTick, metadata={"instrument_id": instrument_id})
-                )
-            else:
-                self._loop.create_task(self._subscribe_mark_price(instrument_id))
-                self._add_subscription(DataType(MarkTick))
-
-    async def _subscribe_mark_price(self, instrument_id: Optional[InstrumentId] = None):
-        if instrument_id:
-            await self._ws_api.subscribe_mark_price(symbol=instrument_id.symbol.value)
-        else:
-            await self._ws_api.subscribe_mark_price()
-
     def subscribe_instruments(self):
         """
         Subscribe to instrument data for the venue.
@@ -224,162 +138,14 @@ class ZbDataClient(LiveMarketDataClient):
         """
         self._add_subscription_instrument(instrument_id)
 
-    def subscribe_order_book_deltas(
-        self,
-        instrument_id: InstrumentId,
-        book_type: BookType,
-        depth: Optional[int] = None,
-        kwargs: dict = None,
-    ):
-        self._loop.create_task(
-            self._subscribe_order_book_deltas(
-                instrument_id=instrument_id,
-                book_type=book_type,
-                depth=depth,
-            )
-        )
-
-        self._add_subscription_order_book_deltas(instrument_id)
-
-    def subscribe_order_book_snapshots(
-        self,
-        instrument_id: InstrumentId,
-        book_type: BookType,
-        depth: Optional[int] = None,
-        kwargs: dict = None,
-    ):
-        self._loop.create_task(
-            self._subscribe_order_book_snapshot(
-                instrument_id=instrument_id,
-                book_type=book_type,
-                depth=depth,
-            )
-        )
-
-        self._add_subscription_order_book_snapshots(instrument_id)
-
-    async def _subscribe_order_book_snapshot(
-        self,
-        instrument_id: InstrumentId,
-        book_type: BookType,
-        depth: Optional[int] = None,
-    ):
-        if book_type == BookType.L3_MBO:
-            self._log.error(
-                "Cannot subscribe to order book deltas: "
-                "L3_MBO data is not published by Zb. "
-                "Valid book types are L1_TBBO, L2_MBP.",
-            )
-            return
-
-        if depth is None or depth == 0:
-            depth = 5
-
-        if depth >= 10:
-            self._log.error(
-                "Cannot subscribe to order book snapshots: "
-                f"invalid depth, was {depth}. "
-                "Valid depths are 1 to 10.",
-            )
-            return
-
-        await self._ws_api.subscribe_book_snapshot(
-            symbol=instrument_id.symbol.value,
-            depth=depth,
-        )
-
-        while not self._ws_api.is_connected:
-            await self.sleep0()
-
-    async def _subscribe_order_book_deltas(
-        self,
-        instrument_id: InstrumentId,
-        book_type: BookType,
-        depth: Optional[int] = None,
-    ):
-        if book_type == BookType.L3_MBO:
-            self._log.error(
-                "Cannot subscribe to order book deltas: "
-                "L3_MBO data is not published by Zb. "
-                "Valid book types are L1_TBBO, L2_MBP.",
-            )
-            return
-
-        if depth is None or depth == 0:
-            depth = 50
-
-        if depth >= 1000:
-            self._log.error(
-                "Cannot subscribe to order book deltas: "
-                f"invalid depth, was {depth}. "
-                "Valid depths are 1 to 1000.",
-            )
-            return
-
-        await self._ws_api.subscribe_book_deltas(
-            symbol=instrument_id.symbol.value,
-            depth=depth,
-        )
-
-        while not self._ws_api.is_connected:
-            await self.sleep0()
-
     def subscribe_ticker(self, instrument_id: InstrumentId):
         self._loop.create_task(self._ws_api.subscribe_ticker(instrument_id.symbol.value))
         self._add_subscription_ticker(instrument_id)
 
-    def subscribe_quote_ticks(self, instrument_id: InstrumentId):
-        self._loop.create_task(
-            self._ws_api.subscribe_book_deltas(
-                symbol=instrument_id.symbol.value,
-                depth=1,
-            )
-        )
-        self._quote_tick_subscribed = True
-        self._add_subscription_quote_ticks(instrument_id)
-
     def subscribe_trade_ticks(self, instrument_id: InstrumentId):
         self._loop.create_task(self._ws_api.subscribe_trades(instrument_id.symbol.value))
         self._add_subscription_trade_ticks(instrument_id)
 
-    def subscribe_bars(self, bar_type: BarType):
-        PyCondition.true(bar_type.is_externally_aggregated(), "aggregation_source is not EXTERNAL")
-
-        if not bar_type.spec.is_time_aggregated():
-            self._log.error(
-                f"Cannot subscribe to {bar_type}: only time bars are aggregated by Zb.",
-            )
-            return
-
-        if bar_type.spec.aggregation == BarAggregation.SECOND:
-            self._log.error(
-                f"Cannot subscribe to {bar_type}: second bars are not aggregated by Zb.",
-            )
-            return
-
-        if bar_type.spec.aggregation == BarAggregation.MINUTE:
-            resolution = "M"
-        elif bar_type.spec.aggregation == BarAggregation.HOUR:
-            resolution = "H"
-        elif bar_type.spec.aggregation == BarAggregation.DAY:
-            resolution = "D"
-        else:  # pragma: no cover (design-time error)
-            raise RuntimeError(
-                f"invalid aggregation period, "
-                f"was {BarAggregationParser.from_str(bar_type.spec.aggregation)}",
-            )
-
-        if bar_type.instrument_id not in self._kline_buffer:
-            self._kline_buffer[bar_type.instrument_id] = []
-
-        self._loop.create_task(
-            self._ws_api.subscribe_bars(
-                symbol=bar_type.instrument_id.symbol.value,
-                interval=f"{bar_type.spec.step}{resolution}",
-            )
-        )
-        self._add_subscription_bars(bar_type)
-
     def subscribe_instrument_status_updates(self, instrument_id: InstrumentId):
         self._log.warning(
             "Cannot subscribe to instrument status updates: "
@@ -450,44 +216,6 @@ class ZbDataClient(LiveMarketDataClient):
             "Cannot request historical quote ticks: not published by Zb.",
         )
 
-    def request_trade_ticks(
-        self,
-        instrument_id: InstrumentId,
-        from_datetime: pd.Timestamp,
-        to_datetime: pd.Timestamp,
-        limit: int,
-        correlation_id: UUID4,
-    ):
-        if limit == 0 or limit > 1000:
-            limit = 1000
-
-        if from_datetime is not None or to_datetime is not None:
-            self._log.warning(
-                "Trade ticks have been requested with a from/to time range, "
-                f"however the request will be for the most recent {limit}."
-            )
-
-        self._loop.create_task(self._request_trade_ticks(instrument_id, limit, correlation_id))
-
-    async def _request_trade_ticks(
-        self,
-        instrument_id: InstrumentId,
-        limit: int,
-        correlation_id: UUID4,
-    ):
-        response: Dict[str, Any] = await self._market_api.trades(instrument_id.symbol.value, limit)
-
-        ticks: List[TradeTick] = [
-            parse_trade_tick(
-                msg=t,
-                instrument_id=instrument_id,
-                ts_init=self._clock.timestamp_ns(),
-            )
-            for t in response["data"]
-        ]
-
-        self._handle_trade_ticks(instrument_id, ticks, correlation_id)
-
     def request_bars(
         self,
         bar_type: BarType,
@@ -532,6 +260,512 @@ class ZbDataClient(LiveMarketDataClient):
             )
         )
 
+    def _send_all_instruments_to_data_engine(self):
+        for instrument in self._instrument_provider.get_all().values():
+            self._handle_data(instrument)
+
+        for currency in self._instrument_provider.currencies().values():
+            self._cache.add_currency(currency)
+
+    def _handle_pong(self):
+        # reset ping timer
+        pass
+
+
+class ZbSpotDataClient(ZbDataClient):
+    def __init__(
+        self,
+        loop: asyncio.AbstractEventLoop,
+        client: ZbSpotHttpClient,
+        msgbus: MessageBus,
+        cache: Cache,
+        clock: LiveClock,
+        logger: Logger,
+        venue: Venue,
+        instrument_provider: ZbInstrumentProvider,
+    ):
+        # HTTP API
+        self._market_api = ZbSpotMarketHttpAPI(client=client)
+        # websocket api
+        self._ws_api = ZbSpotWebSocket(
+            loop=loop,
+            clock=clock,
+            logger=logger,
+            handler=self._handle_ws_message,
+        )
+        super(ZbDataClient, self).__init__(
+            loop=loop,
+            client_id=ClientId(venue.value),
+            instrument_provider=instrument_provider,
+            msgbus=msgbus,
+            cache=cache,
+            clock=clock,
+            logger=logger,
+            config={"name": f"ZbDataClient-{venue.value}"},
+        )
+
+        self._client = client
+
+        self._update_instrument_interval: int = 60 * 60  # Once per hour (hardcode)
+        self._update_instruments_task: Optional[asyncio.Task] = None
+
+    def subscribe_order_book_deltas(
+        self,
+        instrument_id: InstrumentId,
+        book_type: BookType,
+        depth: Optional[int] = None,
+        kwargs: dict = None,
+    ):
+        self._loop.create_task(
+            self._subscribe_order_book_snapshot(
+                instrument_id=instrument_id,
+                book_type=book_type,
+                depth=depth,
+            )
+        )
+
+        self._add_subscription_order_book_deltas(instrument_id)
+
+    def subscribe_order_book_snapshots(
+        self,
+        instrument_id: InstrumentId,
+        book_type: BookType,
+        depth: Optional[int] = None,
+        kwargs: dict = None,
+    ):
+        self._loop.create_task(
+            self._subscribe_order_book_snapshot(
+                instrument_id=instrument_id,
+                book_type=book_type,
+                depth=depth,
+            )
+        )
+
+        self._add_subscription_order_book_snapshots(instrument_id)
+
+    async def _subscribe_order_book_snapshot(
+        self,
+        instrument_id: InstrumentId,
+        book_type: BookType,
+        depth: Optional[int] = None,
+    ):
+        if book_type == BookType.L3_MBO:
+            self._log.error(
+                "Cannot subscribe to order book deltas: "
+                "L3_MBO data is not published by Zb. "
+                "Valid book types are L1_TBBO, L2_MBP.",
+            )
+            return
+
+        if depth is not None and depth != 0:
+            self._log.warning(
+                "Cannot subscribe to depth order book snapshots: "
+                f"invalid depth, was {depth}. "
+                "Zb does not support custom depth.",
+            )
+
+        await self._ws_api.subscribe_depth(
+            symbol=instrument_id.symbol.value,
+        )
+
+    def subscribe_quote_ticks(self, instrument_id: InstrumentId):
+        self._log.warning(
+            "Cannot subscribe to quote tick updates: "
+            "Not currently supported for the Zb spot integration.",
+        )
+
+    def subscribe_bars(self, bar_type: BarType):
+        self._log.error(
+            "Cannot subscribe to bar updates: "
+            "Not currently supported for the Zb spot integration.",
+        )
+
+    def request_trade_ticks(
+        self,
+        instrument_id: InstrumentId,
+        from_datetime: pd.Timestamp,
+        to_datetime: pd.Timestamp,
+        limit: int,
+        correlation_id: UUID4,
+    ):
+        if limit == 0 or limit > 50:
+            limit = 50
+
+        if from_datetime is not None or to_datetime is not None:
+            self._log.warning(
+                "Trade ticks have been requested with a from/to time range, "
+                f"however the request will be for the most recent {limit}."
+            )
+
+        self._loop.create_task(self._request_trade_ticks(instrument_id, limit, correlation_id))
+
+    async def _request_trade_ticks(
+        self,
+        instrument_id: InstrumentId,
+        limit: int,
+        correlation_id: UUID4,
+    ):
+        response: Dict[str, Any] = await self._market_api.trades(instrument_id.symbol.value, limit)
+
+        ticks: List[TradeTick] = [
+            parse_trade_tick(
+                msg=t,
+                instrument_id=instrument_id,
+                ts_init=self._clock.timestamp_ns(),
+            )
+            for t in response["data"]
+        ]
+
+        self._handle_trade_ticks(instrument_id, ticks, correlation_id)
+
+    async def _request_bars(
+        self,
+        bar_type: BarType,
+        from_datetime: pd.Timestamp,
+        to_datetime: pd.Timestamp,
+        limit: int,
+        correlation_id: UUID4,
+    ):
+        if limit == 0 or limit > 1000:
+            limit = 1000
+
+        if bar_type.spec.aggregation == BarAggregation.MINUTE:
+            resolution = "min"
+        elif bar_type.spec.aggregation == BarAggregation.HOUR:
+            resolution = "hour"
+        elif bar_type.spec.aggregation == BarAggregation.DAY:
+            resolution = "day"
+        else:  # pragma: no cover (design-time error)
+            raise RuntimeError(
+                f"invalid aggregation period, "
+                f"was {BarAggregationParser.from_str(bar_type.spec.aggregation)}",
+            )
+
+        if to_datetime is not None:
+            self._log.warning(
+                "Trade ticks have been requested with to_datetime option, "
+                f"however the request will be for the most recent {limit} or after from_datetime."
+            )
+
+        start_time_ms = from_datetime.to_datetime64() * 1000 if from_datetime is not None else None
+
+        data: Dict[str, Any] = await self._market_api.kline(
+            market=bar_type.instrument_id.symbol.value,
+            type=f"{bar_type.spec.step}{resolution}",
+            since=start_time_ms,
+            size=limit,
+        )
+
+        bars: List[ZbBar] = [
+            parse_spot_bar(bar_type, values=b, ts_init=self._clock.timestamp_ns())
+            for b in data["data"]
+        ]
+        partial: ZbBar = bars.pop()
+
+        self._handle_bars(bar_type, bars, partial, correlation_id)
+
+    def _handle_ws_message(self, raw: bytes):
+        try:
+            msg: Dict[str, Any] = orjson.loads(raw)
+        except orjson.JSONDecodeError as ex:
+            if raw.decode() == "pong":
+                self._handle_pong()
+                return
+            else:
+                raise ex
+
+        # data = msg.get("data")
+        chan: Optional[str] = msg.get("channel")
+
+        if not chan:
+            self._log.error(f"Unknown response: {msg}")
+            return
+
+        symbol, _, chan_type = chan.partition("_")
+
+        if chan_type == "ticker":
+            self._handle_24hr_ticker(msg["ticker"], symbol)
+        elif chan_type == "trades":
+            self._handle_trade(msg["data"], symbol)
+        elif chan_type == "depth":
+            self._handle_book_snapshot(msg, symbol)
+        else:
+            self._log.error(f"Unrecognized websocket message type, was {chan}")
+            return
+
+    def _handle_24hr_ticker(self, data: Dict[str, str], symbol: str):
+        instrument_id = self._instrument_provider.find_instrument_id_by_local_market_id(symbol)
+        ticker: ZbTicker = parse_spot_ticker_ws(
+            instrument_id=instrument_id,
+            msg=data,
+            ts_init=self._clock.timestamp_ns(),
+        )
+        self._handle_data(ticker)
+
+    def _handle_trade(self, data: List[Dict[str, str]], symbol: str):
+        instrument_id = self._instrument_provider.find_instrument_id_by_local_market_id(symbol)
+        for d in data:
+            trade_tick: TradeTick = parse_spot_trade_tick_ws(
+                instrument_id=instrument_id,
+                msg=d,
+                ts_init=self._clock.timestamp_ns(),
+            )
+            self._handle_data(trade_tick)
+
+    def _handle_book_snapshot(
+        self,
+        data: Dict,
+        symbol: str,
+    ):
+        instrument_id = self._instrument_provider.find_instrument_id_by_local_market_id(symbol)
+        book_snapshot: OrderBookSnapshot = parse_spot_book_snapshot_ws(
+            instrument_id=instrument_id,
+            msg=data,
+            ts_init=self._clock.timestamp_ns(),
+        )
+        self._handle_data(book_snapshot)
+
+
+class ZbFutureDataClient(ZbDataClient):
+    def __init__(
+        self,
+        loop: asyncio.AbstractEventLoop,
+        client: ZbHttpClient,
+        msgbus: MessageBus,
+        cache: Cache,
+        clock: LiveClock,
+        logger: Logger,
+        venue: Venue,
+        instrument_provider: ZbInstrumentProvider,
+    ):
+        # HTTP API
+        self._market_api = ZbFutureMarketHttpAPI(client=client)
+        # websocket api
+        self._ws_api = ZbFuturesWebSocket(
+            loop=loop,
+            clock=clock,
+            logger=logger,
+            handler=self._handle_ws_message,
+        )
+
+        super(ZbDataClient, self).__init__(
+            loop=loop,
+            client_id=ClientId(venue.value),
+            instrument_provider=instrument_provider,
+            msgbus=msgbus,
+            cache=cache,
+            clock=clock,
+            logger=logger,
+            config={"name": f"ZbDataClient-{venue.value}"},
+        )
+
+        self._client = client
+
+        self._update_instrument_interval: int = 60 * 60  # Once per hour (hardcode)
+        self._update_instruments_task: Optional[asyncio.Task] = None
+
+        self._kline_buffer: Dict[InstrumentId, List[Any]] = {}
+        self._quote_tick_subscribed = False
+        self._last_quote_tick: Dict[InstrumentId, QuoteTick] = {}
+
+    def subscribe(self, data_type: DataType) -> None:
+        if data_type.type == MarkTick:
+            instrument_id = data_type.metadata.get("instrument_id")
+            if instrument_id:
+                self._loop.create_task(self._subscribe_mark_price())
+                self._add_subscription(
+                    DataType(MarkTick, metadata={"instrument_id": instrument_id})
+                )
+            else:
+                self._loop.create_task(self._subscribe_mark_price(instrument_id))
+                self._add_subscription(DataType(MarkTick))
+
+    async def _subscribe_mark_price(self, instrument_id: Optional[InstrumentId] = None):
+        if instrument_id:
+            await self._ws_api.subscribe_mark_price(symbol=instrument_id.symbol.value)
+        else:
+            await self._ws_api.subscribe_mark_price()
+
+    def subscribe_quote_ticks(self, instrument_id: InstrumentId):
+        self._loop.create_task(
+            self._ws_api.subscribe_book_deltas(
+                symbol=instrument_id.symbol.value,
+                depth=1,
+            )
+        )
+        self._quote_tick_subscribed = True
+        self._add_subscription_quote_ticks(instrument_id)
+
+    def subscribe_bars(self, bar_type: BarType):
+        PyCondition.true(bar_type.is_externally_aggregated(), "aggregation_source is not EXTERNAL")
+
+        if not bar_type.spec.is_time_aggregated():
+            self._log.error(
+                f"Cannot subscribe to {bar_type}: only time bars are aggregated by Zb.",
+            )
+            return
+
+        if bar_type.spec.aggregation == BarAggregation.SECOND:
+            self._log.error(
+                f"Cannot subscribe to {bar_type}: second bars are not aggregated by Zb.",
+            )
+            return
+
+        if bar_type.spec.aggregation == BarAggregation.MINUTE:
+            resolution = "M"
+        elif bar_type.spec.aggregation == BarAggregation.HOUR:
+            resolution = "H"
+        elif bar_type.spec.aggregation == BarAggregation.DAY:
+            resolution = "D"
+        else:  # pragma: no cover (design-time error)
+            raise RuntimeError(
+                f"invalid aggregation period, "
+                f"was {BarAggregationParser.from_str(bar_type.spec.aggregation)}",
+            )
+
+        if bar_type.instrument_id not in self._kline_buffer:
+            self._kline_buffer[bar_type.instrument_id] = []
+
+        self._loop.create_task(
+            self._ws_api.subscribe_bars(
+                symbol=bar_type.instrument_id.symbol.value,
+                interval=f"{bar_type.spec.step}{resolution}",
+            )
+        )
+        self._add_subscription_bars(bar_type)
+
+    def subscribe_order_book_deltas(
+        self,
+        instrument_id: InstrumentId,
+        book_type: BookType,
+        depth: Optional[int] = None,
+        kwargs: dict = None,
+    ):
+        self._loop.create_task(
+            self._subscribe_order_book_deltas(
+                instrument_id=instrument_id,
+                book_type=book_type,
+                depth=depth,
+            )
+        )
+
+        self._add_subscription_order_book_deltas(instrument_id)
+
+    def subscribe_order_book_snapshots(
+        self,
+        instrument_id: InstrumentId,
+        book_type: BookType,
+        depth: Optional[int] = None,
+        kwargs: dict = None,
+    ):
+        self._loop.create_task(
+            self._subscribe_order_book_snapshot(
+                instrument_id=instrument_id,
+                book_type=book_type,
+                depth=depth,
+            )
+        )
+
+        self._add_subscription_order_book_snapshots(instrument_id)
+
+    async def _subscribe_order_book_snapshot(
+        self,
+        instrument_id: InstrumentId,
+        book_type: BookType,
+        depth: Optional[int] = None,
+    ):
+        if book_type == BookType.L3_MBO:
+            self._log.error(
+                "Cannot subscribe to order book deltas: "
+                "L3_MBO data is not published by Zb. "
+                "Valid book types are L1_TBBO, L2_MBP.",
+            )
+            return
+
+        if depth is None or depth == 0:
+            depth = 5
+
+        if depth > 10:
+            self._log.error(
+                "Cannot subscribe to order book snapshots: "
+                f"invalid depth, was {depth}. "
+                "Valid depths are 1 to 10.",
+            )
+            return
+
+        await self._ws_api.subscribe_book_snapshot(
+            symbol=instrument_id.symbol.value,
+            depth=depth,
+        )
+
+    async def _subscribe_order_book_deltas(
+        self,
+        instrument_id: InstrumentId,
+        book_type: BookType,
+        depth: Optional[int] = None,
+    ):
+        if book_type == BookType.L3_MBO:
+            self._log.error(
+                "Cannot subscribe to order book deltas: "
+                "L3_MBO data is not published by Zb. "
+                "Valid book types are L1_TBBO, L2_MBP.",
+            )
+            return
+
+        if depth is None or depth == 0:
+            depth = 50
+
+        if depth >= 1000:
+            self._log.error(
+                "Cannot subscribe to order book deltas: "
+                f"invalid depth, was {depth}. "
+                "Valid depths are 1 to 1000.",
+            )
+            return
+
+        await self._ws_api.subscribe_book_deltas(
+            symbol=instrument_id.symbol.value,
+            depth=depth,
+        )
+
+    def request_trade_ticks(
+        self,
+        instrument_id: InstrumentId,
+        from_datetime: pd.Timestamp,
+        to_datetime: pd.Timestamp,
+        limit: int,
+        correlation_id: UUID4,
+    ):
+        if limit == 0 or limit > 1000:
+            limit = 1000
+
+        if from_datetime is not None or to_datetime is not None:
+            self._log.warning(
+                "Trade ticks have been requested with a from/to time range, "
+                f"however the request will be for the most recent {limit}."
+            )
+
+        self._loop.create_task(self._request_trade_ticks(instrument_id, limit, correlation_id))
+
+    async def _request_trade_ticks(
+        self,
+        instrument_id: InstrumentId,
+        limit: int,
+        correlation_id: UUID4,
+    ):
+        response: Dict[str, Any] = await self._market_api.trades(instrument_id.symbol.value, limit)
+
+        ticks: List[TradeTick] = [
+            parse_trade_tick(
+                msg=t,
+                instrument_id=instrument_id,
+                ts_init=self._clock.timestamp_ns(),
+            )
+            for t in response["data"]
+        ]
+
+        self._handle_trade_ticks(instrument_id, ticks, correlation_id)
+
     async def _request_bars(
         self,
         bar_type: BarType,
@@ -574,13 +808,6 @@ class ZbDataClient(LiveMarketDataClient):
 
         self._handle_bars(bar_type, bars, partial, correlation_id)
 
-    def _send_all_instruments_to_data_engine(self):
-        for instrument in self._instrument_provider.get_all().values():
-            self._handle_data(instrument)
-
-        for currency in self._instrument_provider.currencies().values():
-            self._cache.add_currency(currency)
-
     def _handle_ws_message(self, raw: bytes):  # noqa: C901
         msg: Dict[str, Any] = orjson.loads(raw)
         data = msg.get("data")
@@ -629,10 +856,6 @@ class ZbDataClient(LiveMarketDataClient):
             self._log.error(f"Unrecognized websocket message type, was {chan}")
             return
 
-    def _handle_pong(self):
-        # reset ping timer
-        pass
-
     def _handle_book_snapshot(
         self,
         data: Dict[str, Any],
@@ -794,75 +1017,3 @@ class ZbDataClient(LiveMarketDataClient):
         msg_type: Optional[str] = None,
     ):
         pass
-
-
-class ZbSpotDataClient(ZbDataClient):
-    def __init__(
-        self,
-        loop: asyncio.AbstractEventLoop,
-        client: ZbHttpClient,
-        msgbus: MessageBus,
-        cache: Cache,
-        clock: LiveClock,
-        logger: Logger,
-        venue: Venue,
-        instrument_provider: ZbInstrumentProvider,
-    ):
-        # HTTP API
-        market_api = ZbSpotMarketHttpAPI(client=client)
-        # websocket api
-        ws_api = ZbSpotWebSocket(
-            loop=loop,
-            clock=clock,
-            logger=logger,
-            handler=self._handle_ws_message,
-        )
-
-        super().__init__(
-            loop=loop,
-            client=client,
-            msgbus=msgbus,
-            cache=cache,
-            clock=clock,
-            logger=logger,
-            venue=venue,
-            market_api=market_api,
-            ws_api=ws_api,
-            instrument_provider=instrument_provider,
-        )
-
-
-class ZbFutureDataClient(ZbDataClient):
-    def __init__(
-        self,
-        loop: asyncio.AbstractEventLoop,
-        client: ZbHttpClient,
-        msgbus: MessageBus,
-        cache: Cache,
-        clock: LiveClock,
-        logger: Logger,
-        venue: Venue,
-        instrument_provider: ZbInstrumentProvider,
-    ):
-        # HTTP API
-        market_api = ZbFutureMarketHttpAPI(client=client)
-        # websocket api
-        ws_api = ZbFuturesWebSocket(
-            loop=loop,
-            clock=clock,
-            logger=logger,
-            handler=self._handle_ws_message,
-        )
-
-        super().__init__(
-            loop=loop,
-            client=client,
-            msgbus=msgbus,
-            cache=cache,
-            clock=clock,
-            logger=logger,
-            venue=venue,
-            market_api=market_api,
-            ws_api=ws_api,
-            instrument_provider=instrument_provider,
-        )
