@@ -19,6 +19,7 @@ import platform
 import socket
 import sys
 import time
+import warnings
 from datetime import timedelta
 from typing import Any, List, Optional
 
@@ -49,19 +50,32 @@ from nacre.msgbus.msgbus_ext import MessageBusExt
 from nacre.msgbus.pubsub import PubSub
 
 
+try:
+    import uvloop
+
+    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
+    uvloop_version = uvloop.__version__
+except ImportError:  # pragma: no cover
+    uvloop_version = None
+    warnings.warn("uvloop is not available.")
+
+
 class TradingNode(NautilusTradingNode):
+    """
+    Provides an asynchronous network node for live trading.
+
+    Parameters
+    ----------
+    config : TradingNodeConfig, optional
+        The configuration for the instance.
+
+    Raises
+    ------
+    TypeError
+        If `config` is not of type `TradingNodeConfig`.
+    """
+
     def __init__(self, config: Optional[TradingNodeConfig] = None):  # noqa: C901
-        """
-        Initialize a new instance of the TradingNode class.
-        Parameters
-        ----------
-        config : TradingNodeConfig, optional
-            The configuration for the instance.
-        Raises
-        ------
-        TypeError
-            If `config` is not of type `TradingNodeConfig`.
-        """
         if config is None:
             config = TradingNodeConfig()
         PyCondition.not_none(config, "config")
@@ -128,12 +142,6 @@ class TradingNode(NautilusTradingNode):
                 "can one of {{'in-memory', 'redis'}}.",
             )
 
-        self._cache = Cache(
-            database=cache_db,
-            logger=self._logger,
-            config=config.cache,
-        )
-
         ########################################################################
         # Add pubsub
         ########################################################################
@@ -169,6 +177,16 @@ class TradingNode(NautilusTradingNode):
 
         self._pubsub = pubsub
 
+        self._cache = Cache(
+            database=cache_db,
+            logger=self._logger,
+            config=config.cache,
+        )
+
+        ########################################################################
+        # Added
+        ########################################################################
+
         ########################################################################
         # Add exposer
         ########################################################################
@@ -431,6 +449,29 @@ class TradingNode(NautilusTradingNode):
         except asyncio.CancelledError as ex:
             self._log.error(str(ex))
 
+    async def _await_engines_connected(self) -> bool:
+        # - The data engine clients will be set connected when all
+        # instruments are received and updated with the data engine.
+        # - The execution engine clients will be set connected when all
+        # accounts are updated and the current order and position status is
+        # reconciled.
+        # Thus any delay here will be due to blocking network I/O.
+        seconds = self._config.timeout_connection
+        timeout: timedelta = self._clock.utc_now() + timedelta(seconds=seconds)
+        while True:
+            await asyncio.sleep(0)
+            if self._clock.utc_now() >= timeout:
+                return False
+            if not self._data_engine.check_connected():
+                continue
+            if not self._exec_engine.check_connected():
+                continue
+            if self._pubsub and not self._pubsub.check_connected():
+                continue
+            break
+
+        return True  # Engines connected
+
     async def _stop(self) -> None:  # noqa: C901
         self._is_stopping = True
         self._log.info("STOPPING...")
@@ -495,29 +536,6 @@ class TradingNode(NautilusTradingNode):
         self._logger.stop()
         self._is_running = False
 
-    async def _await_engines_connected(self) -> bool:
-        # - The data engine clients will be set connected when all
-        # instruments are received and updated with the data engine.
-        # - The execution engine clients will be set connected when all
-        # accounts are updated and the current order and position status is
-        # reconciled.
-        # Thus any delay here will be due to blocking network I/O.
-        seconds = self._config.timeout_connection
-        timeout: timedelta = self._clock.utc_now() + timedelta(seconds=seconds)
-        while True:
-            await asyncio.sleep(0)
-            if self._clock.utc_now() >= timeout:
-                return False
-            if not self._data_engine.check_connected():
-                continue
-            if not self._exec_engine.check_connected():
-                continue
-            if self._pubsub and not self._pubsub.check_connected():
-                continue
-            break
-
-        return True  # Engines connected
-
     async def _await_engines_disconnected(self) -> bool:
         seconds = self._config.timeout_disconnection
         timeout: timedelta = self._clock.utc_now() + timedelta(seconds=seconds)
