@@ -1,6 +1,5 @@
 from nautilus_trader.model.events.account import AccountState
-
-from nacre.serialization.avro.patch_registry import schema
+from schema_registry.client import schema
 
 
 NAUTILUS_AVRO_SCHEMA = {
@@ -20,6 +19,7 @@ NAUTILUS_AVRO_SCHEMA = {
                 {"name": "ts_event", "type": "long"},
                 {"name": "ts_init", "type": "long"},
                 {"name": "equity", "type": "float"},
+                {"name": "equities", "type": "string", "default": "[]"},
             ],
         }
     ),
